#!/bin/sh 

dprint() {
  echo $1 > /dev/kmsg
}

/usr/bin/sys_stat.sh set_status_led green_flash

# note sed will break soft link, so use real path
ISSDIR=/usr/local/bin
ISSCONF="$ISSDIR/iss.conf"
ISSNEXT_CONF="$ISSDIR/iss-next.conf"
CUSTCONF="$ISSDIR/isscust.conf"
ISSDEF="$ISSDIR/iss-default.conf"
CUSTDEF="$ISSDIR/isscust-default.conf"
NVRAMFN="$ISSDIR/issnvram.txt"
NVRAMFN_DEF="$ISSDIR/issnvram_def.txt"

ISS_BIN="$ISSDIR/ISS408wt.exe"
BACKEND_BIN="$ISSDIR/aricent-backend.arm"
UPDATE_FLAG="$ISSDIR/update.flag"

DEVMEM=/sbin/devmem
AC3_DATA_OUT_SET=0xf1018130
AC3_DATA_OUT_CLEAR=0xf1018134

tmp_mac=`i2ctool get_mac`
tmp_mac_len=${#tmp_mac}
tmp_serial=`i2ctool get_serial`
tmp_serial_len=${#tmp_serial}
tmp_model70=`i2ctool get_model70`
tmp_model70_len=${#tmp_model70}
tmp_model=`i2ctool get_model`
if [ $tmp_mac_len -ne 12 ]; then
	sleep 20
	dprint "Please run /etc/reg_boardinfo.sh to configure board info"
	exit 0
#elif [ $tmp_serial_len -ne 10 ]; then
#	sleep 20
#	echo "Please run /etc/reg_boardinfo.sh to configure board info" > /dev/kmsg
#	exit 0
elif [ $tmp_model70_len -ne 12 ]; then
	if [ -z $tmp_model ]; then
		sleep 20
		dprint "Please run /etc/reg_boardinfo.sh to configure board info"
		exit 0
	elif [ $tmp_model == "M408-4C" ]; then
		i2ctool set_model70 70-0Q0340120
	elif [ $tmp_model == "M408-2C" ]; then
		i2ctool set_model70 70-0Q03S0110
	elif [ $tmp_model == "M408S" ]; then
		i2ctool set_model70 70-0Q03T0110
	fi
fi

[ -f /usr/bin/genenvs.sh ] && . /usr/bin/genenvs.sh
if [ -z "$BoardType" ]; then
	BoardType=`i2ctool get_model`
fi
if [ -z "$MacAddr" ]; then
	MacAddr=`i2ctool get_mac_addr`
fi	

# select rtc
/usr/sbin/i2cdetect -y 0 0x30 0x31| grep "30 31" > /dev/null 2>&1
if [ $? -eq 0 ]; then
	# use s35390a rtc
	echo "s35390a 0x30" > /sys/class/i2c-dev/i2c-0/device/new_device
else 	
	/usr/sbin/i2cdetect -y 0 0x32 0x32| grep "32" > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		# use rx8010 rtc
		echo "rx8010 0x32" > /sys/class/i2c-dev/i2c-0/device/new_device
	fi
fi

#in case it's not deleted
rm -f /usr/tmp/firmware.tar.xz;sync
# only first boot or fw update boot need untar binary.tar.xz
if [ ! -f $ISS_BIN ] || [ ! -f $BACKEND_BIN ] || [ -f $UPDATE_FLAG ]; then
	dprint "Unpacking binaries"
	cd /usr/local/ && tar -xf bin.tar.xz
	[ -f $UPDATE_FLAG ] && rm $UPDATE_FLAG
	sync
else
	# wait for openwrt service complete
	sleep 20
fi

mkdir -p /var/lib/
touch /var/lib/logrotate.status
dprint "maintain log files"
rm -f /usr/var/*.gz
/usr/sbin/logrotate /etc/logrotate.conf 
dprint "$(du -hs /usr/var)"
/usr/sbin/crond

/sbin/hwclock -s -f /dev/rtc1

if [ "$QSW_SLEDTYPE" == "SINGLE" ]; then
	# GPIO 1x PIN direction
	/usr/sbin/i2cset -y 0 0x4e 0x10 0x49
	# enable output drive
	/usr/sbin/i2cset -y 0 0x4e 0x1b 0x49
	# GPIO 2x PIN direction
	/usr/sbin/i2cset -y 0 0x4e 0x20 0xf2
	# enable output dr
	/usr/sbin/i2cset -y 0 0x4e 0x2b 0xf2
elif [ "$QSW_SLEDTYPE" == "DUAL" ]; then
	# GPIO 1x PIN direction
	/usr/sbin/i2cset -y 0 0x4e 0x10 0xc9
	# enable output drive
	/usr/sbin/i2cset -y 0 0x4e 0x1b 0xc9
	# GPIO 2x PIN direction
	/usr/sbin/i2cset -y 0 0x4e 0x20 0xf2
	# enable output dr
	/usr/sbin/i2cset -y 0 0x4e 0x2b 0xf2
fi

# select temp2 reg
/usr/sbin/i2cset -y 0 0x2f 0x22 0x8

if [ "$BoardType" == "M408-4C" ] || [ "$BoardType" == "M408-2C" ]; then
	# set fan2 pwm  invert
	/usr/sbin/i2cset -y 0 0x2f 0x5f 0x20
	# set fan speed 800 rpm
	/usr/sbin/i2cset -y 0 0x2f 0x61 0x48
	# smart fan
	# select FANCTL2
	/usr/sbin/i2cset -y 0 0x2f 0x64 0x20
	# select temp2
	/usr/sbin/i2cset -y 0 0x2f 0x66 0x90
	#60,75,90,100 degree C
	/usr/sbin/i2cset -y 0 0x2f 0x90 0x3c
	/usr/sbin/i2cset -y 0 0x2f 0x91 0x4b
	/usr/sbin/i2cset -y 0 0x2f 0x92 0x5a
	/usr/sbin/i2cset -y 0 0x2f 0x93 0x64
	#critial temp 110 degree C
	/usr/sbin/i2cset -y 0 0x2f 0x94 0x6e
	#duty cycle 20%,30%,40%,50%
	#/usr/sbin/i2cset -y 0 0x2f 0x95 0x33
	/usr/sbin/i2cset -y 0 0x2f 0x95 0x4c
	/usr/sbin/i2cset -y 0 0x2f 0x96 0x4d
	/usr/sbin/i2cset -y 0 0x2f 0x97 0x66
	/usr/sbin/i2cset -y 0 0x2f 0x98 0x80	
	
	# temp2 critical hyst5, opertion hyst 5
	/usr/sbin/i2cset -y 0 0x2f 0x75 0x55
fi

modprobe mvDmaDrv
modprobe mvIntDrv 
modprobe mvMbusDrv

if [ ! -f /etc/iss.config ]; then
	if [ "$BoardType" == "M408-4C" ] || [ "$BoardType" == "M408-2C" ] || [ "$BoardType" == "M408S" ]; then
		/usr/bin/setmode.sh user
	else
		ln -s /etc/M408-4C.config /etc/iss.config
	fi	
fi

# the config partition risks data loss.  We migrate them to /usr/local/bin
migrate_link() {
    for linkfile in "$@"
    do
        if [ -L $linkfile  ]; then
            dprint  "migrating $linkfile"
            actualf=$(readlink $linkfile)
            rm -f $linkfile
            cp $actualf $linkfile
         fi
    done
}

if [ -f /etc/crepo.sh ]; then
  . /etc/crepo.sh
elif [ -f /lib/functions/crepo.sh ]; then
  . /lib/functions/crepo.sh
fi
RMARKF=/etc/.mark_resetconf
headfn=$(crepo_headfn)
preprocess_configs() {
  # import iss.conf
  if [ -f $ISSNEXT_CONF ]; then
    dprint "Use imported ISS config"
    mv $ISSNEXT_CONF $ISSCONF
    crepo_push_file $ISSCONF
  elif [ -f $RMARKF ]; then
    rm $RMARKF
    dprint "Reset ISS config to default"
    cp $ISSDEF $ISSCONF
    confset $ISSCONF 1.3.6.1.4.1.2076.81.1.1.0 QSW-${tmp_model}
    crepo_push_file $ISSCONF
  elif ! cmp $ISSCONF $headfn; then
    dprint "Config lost/taint! Restore from crepo"
    cp $headfn $ISSCONF
  else
    dprint "Config looks fine"
  fi

  # iss-cust.conf
  if [ ! -s $CUSTCONF ]; then
    dprint "Reset ISS custom config to default"
    cp $CUSTDEF $CUSTCONF
    echo "SwitchName=QSW-${tmp_model}" >>  $CUSTCONF
  fi

  # nvram
  if [ ! -s $NVRAMFN ]; then
    dprint "ISS NVRAM file is lost.  Recreate it."
    cp $NVRAMFN_DEF $NVRAMFN
  fi
  sync
}

migrate_link $ISSCONF $CUSTCONF $NVRAMFN $ISSDEF $CUSTDEF
preprocess_configs


# set mac addr to iss	
MacLen=`expr length "$MacAddr"`	
if [ 17 -ne $MacLen ]; then
		addr=`hexdump -vn 5 -e ' /1 "%02X"' /dev/urandom`
		MacAddr="00"$addr
else
	userfn="/usr/local/bin/users"
	if [ ! -f "$userfn" ]; then
		Mac=`echo $MacAddr | awk -F':' '{print $1 $2 $3 $4 $5 $6}'`
		/usr/bin/passgen $Mac > $userfn
	fi		
fi

sed -i "s/SWITCH_MAC.*/SWITCH_MAC =$MacAddr/" $NVRAMFN
SwitchName=`grep SwitchName  $CUSTCONF | cut -d= -f2`
sed -i "s/SWITCH_NAME.*/SWITCH_NAME = $SwitchName/" $NVRAMFN

sync	

mainprog() {
  ifconfig br-lan down
  /usr/local/bin/issmon.sh &
  if [ -f /etc/vlan.config ]; then
     sleep 10
     /etc/setvlans.sh
  fi
}

if [ ! -e /etc/develmode ]; then
  mainprog
fi
# button event daemon
/usr/bin/eventd

# set 1g led 
$DEVMEM $AC3_DATA_OUT_SET 32 0x20
$DEVMEM $AC3_DATA_OUT_SET 32 0x40
$DEVMEM $AC3_DATA_OUT_SET 32 0x100
$DEVMEM $AC3_DATA_OUT_SET 32 0x200
$DEVMEM $AC3_DATA_OUT_SET 32 0x20000
$DEVMEM $AC3_DATA_OUT_SET 32 0x800
$DEVMEM $AC3_DATA_OUT_SET 32 0x1000
$DEVMEM $AC3_DATA_OUT_SET 32 0x2000

if [ -f /nand_config_crash ]; then
	dprint "recovering crash conf volume"
	# detach
	ubidetach /dev/ubi_ctrl -m 4
	# format
	ubiformat /dev/mtd4 -y -q
	# attach
	ubiattach -m 4 /dev/ubi_ctrl
	# create volume
	ubimkvol -N nand_config -m /dev/ubi0 
	mkdir -p /nand_config
	/bin/mount -t ubifs ubi0:nand_config /nand_config
	mkdir -p /nand_config/boot
	echo 1 > /nand_config/boot/boot
	echo 1 > /nand_config/boot/partition	
	cp -a /.etc.lower /nand_config/.etc.upper
	sync
	umount /nand_config/ 
	rmdir /nand_config/
	rm /nand_config_crash
	sync
fi	

dprint "start service completed"

exit 0
